{*******************************************************
 Class:       TOTA
 Created      8. 1998.
 Author:      Martin Waldenburg
 CopyRight:   Martin Waldenburg 1998 all rights reserved
 Description: Wrapper around the ToolServices of Delphi 4.
              You can it use only within a wizard.
              You need not to create an instance to use it.
              To access properties make a global declaration
              Like "OTA: TOTA"  and call for examle OTA.FileName.
              For access to the ClasMethods simple call TOTA.XYMethod.

 Version:     1.4 beta  // will remain in beta state as long as the
                        // new interfaces are so buggy.
 Status:      FreeWare
 Disclaimer:  This is provided as is, expressly without a
              warranty of any kind.
              You use it at your own risc.

 Thanks to Ray Lischner for discovering the trick with the typecast.
 Thanks to Sergey Orlik for some suggestions.
 *******************************************************}

unit mwOTA;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, ActnList, Menus,
  ImgList, comctrls, ActiveX, ToolsApi, DsgnIntf, TypInfo, Forms, mwProp;

type
  TOTA = class(TObject)
  private
  protected
  public
  {************ methods belonging to properties *******************************}
    { The new ToolServices }
    class function GetActionServices: IOTAActionServices;
    class function GetDebuggerServices: IOTADebuggerServices;
    class function GetMessageServices: IOTAMessageServices;
    class function GetModuleServices: IOTAModuleServices;
    class function GetPackageServices: IOTAPackageServices;
    class function GetNTAServices: INTAServices;
    class function GetServices: IOTAServices;
    class function GetWizardServices: IOTAWizardServices;

    { IOTAModuleServices }
    class function GetCurrentModule: IOTAModule;
    class function GetModuleCount: Integer;
    class function GetModule(Index: Integer): IOTAModule;

    { IOTADebuggerServices }
    class function GetAddressBkptCount: Integer;
    class function GetAddressBkpt(Index: Integer): IOTAAddressBreakpoint;
    class function GetCurrentProcess: IOTAProcess;
    class function GetProcessCount: Integer;
    class function GetProcess(Index: Integer): IOTAProcess;
    class function GetSourceBkptCount: Integer;
    class function GetSourceBkpt(Index: Integer): IOTASourceBreakpoint;
    class procedure SetCurrentProcess(Process: IOTAProcess);

   { INTAServices }
    class function AddMasked(Image: TBitmap; MaskColor: TColor): Integer;
    class function GetActionList: TCustomActionList;
    class function GetImageList: TCustomImageList;
    class function GetMainMenu: TMainMenu;
    class function GetToolBar(const ToolBarName: string): TToolBar;

    { IOTAPackageServices }
    class function GetPackageCount: Integer;
    class function GetPackageName(Index: Integer): string;
    class function GetComponentCount(PkgIndex: Integer): Integer;
    class function GetComponentName(PkgIndex, CompIndex: Integer): string;

    { Current IOTAModule }
    class function GetFileName: string;
    class function GetFileSystem: string;
    class function GetOwnerCount: Integer;
    class function GetOwner(Index: Integer): IOTAProject;
    class procedure SetFileName(const AFileName: string);
    class procedure SetFileSystem(const AFileSystem: string);

    { ProjectGroup }
    class function GetProjectGroup: IOTAProjectGroup;
    class function GetActiveProject: IOTAProject;
    class function GetProjectCount: Integer;
    class function GetProject(Index: Integer): IOTAProject;
    class procedure SetActiveProject(const AProject: IOTAProject);

    { SourceEditor }
    class function GetSourceFileName: string;
    class function GetSourceModified: Boolean;
    class function GetSourceModule: IOTAModule;
    class function GetEditViewCount: Integer;
    class function GetEditView(Index: Integer): IOTAEditView;
    class function GetBlockAfter: TOTACharPos;
    class function GetBlockStart: TOTACharPos;
    class function GetBlockType: TOTABlockType;
    class function GetBlockVisible: Boolean;
    class procedure SetBlockAfter(const Value: TOTACharPos);
    class procedure SetBlockStart(const Value: TOTACharPos);
    class procedure SetBlockType(Value: TOTABlockType);
    class procedure SetBlockVisible(Value: Boolean);

    { FormEditor }
    class function GetFormFileName: string;
    class function GetFormModified: Boolean;
    class function GetFormModule: IOTAModule;

    { NTAFormEditor }
    class function GetFormDesigner: IFormDesigner;

    { Current FormDesigner }
    class function GetAncestorDesigner: IFormDesigner;
    class function GetIsDormant: Boolean;

    { Current Project }
    class function GetProjectOptions: IOTAProjectOptions;
    class function GetProjectBuilder: IOTAProjectBuilder;

    { Current ProjectBuilder }
    class function GetShouldBuild: Boolean;

    { Current Process }
    class function GetCurrentThread: IOTAThread;
    class procedure SetCurrentThread(Value: IOTAThread);
    class function GetProcessId: LongWord;
    class function GetThreadCount: Integer;
    class function GetThread(Index: Integer): IOTAThread;

    { Current Thread }
    class function GetCallCount: Integer;
    class function GetCallHeader(Index: Integer): string;
    class function GetCurrentFile: string;
    class function GetCurrentLine: LongWord;
    class function GetContext: TContext;
    class function GetHandle: THandle;
    class function GetState: TOTAThreadState;

    { The current EditWriter }
    class function GetCurrentPos: TOTACharPos;

    { The EditView at index 0 of the current editor }
    class function GetCursorPos: TOTAEditPos;
    class procedure SetCursorPos(const Value: TOTAEditPos);
    class function GetTopPos: TOTAEditPos;
    class procedure SetTopPos(const Value: TOTAEditPos);
    class function GetViewSize: TSize;

  {****************************************************************************}

    { IOTAActionServices }
    class function CloseFile(const FileName: string): Boolean;
    class function OpenFile(const FileName: string): Boolean;
    class function OpenProject(const ProjName: string; NewProjGroup: Boolean): Boolean;
    class function ReloadFile(const FileName: string): Boolean;
    class function SaveFile(const FileName: string): Boolean;

    { IOTAModuleServices }
    class function AddFileSystem(FileSystem: IOTAFileSystem): Integer;
    class function CloseAll: Boolean;
    class function CreateModule(const Creator: IOTACreator): IOTAModule;
    class function FindFileSystem(const Name: string): IOTAFileSystem;
    class function FindFormModule(const FormName: string): IOTAModule;
    class function FindModule(const FileName: string): IOTAModule;
    class procedure GetNewModuleAndClassName(const Prefix: string; var UnitIdent,
      Name, FileName: string);
    class function NewModule: Boolean;
    class procedure RemoveFileSystem(Index: Integer);
    class function SaveAll: Boolean;

    { IOTADebuggerServices }
    class function AddNotifier(const Notifier: IOTADebuggerNotifier): Integer; overload;
    class procedure AttachProcess(Pid: Integer; const RemoteHost: string = '');
    class procedure CreateProcess(const ExeName, Args: string; const RemoteHost: string = '');
    class procedure EnumerateRunningProcesses(Callback: TEnumerateProcessesCallback;
      Param: Pointer; const HostName: string = '');
    class procedure LogString(const LogStr: string);
    class function NewAddressBreakpoint(Address, Length: LongWord; AccessType: TOTAAccessType;
      AProcess: IOTAProcess = nil): IOTABreakpoint;
    class function NewModuleBreakpoint(const ModuleName: string;
      AProcess: IOTAProcess): IOTABreakpoint;
    class function NewSourceBreakpoint(const FileName: string; LineNumber: Integer;
      AProcess: IOTAProcess): IOTABreakpoint;
    class procedure RemoveDebuggerNotifier(Index: Integer);

    { IOTAWizardServices }
    class function AddWizard(const AWizard: IOTAWizard): Integer;
    class procedure RemoveWizard(Index: Integer);

    { IOTAMessageServices }
    class procedure AddCustomMessage(const CustomMsg: IOTACustomMessage);
    class procedure AddTitleMessage(const MessageStr: string);
    class procedure AddToolMessage(const FileName, MessageStr, PrefixStr: string;
      LineNumber, ColumnNumber: Integer);
    class procedure ClearAllMessages;
    class procedure ClearCompilerMessages;
    class procedure ClearSearchMessages;
    class procedure ClearToolMessages;

    { IOTAServices }
    class function AddNotifier(const Notifier: IOTAIDENotifier): Integer; overload;
    class procedure RemoveIDENotifier(Index: Integer);
    class function GetBaseRegistryKey: string;
    class function GetProductIdentifier: string;
    class function GetParentHandle: HWND;
    class function GetEnvironmentOptions: IOTAEnvironmentOptions;

    { Current IOTAModule }
    class function AddNotifier(const ANotifier: IOTAModuleNotifier): Integer; overload;
    class procedure AddToInterface; overload;
    class function Close: Boolean;
    class function GetModuleFileCount: Integer;
    class function GetModuleFileEditor(Index: Integer): IOTAEditor;
    class function HasCoClasses: Boolean;
    class procedure RemoveModuleNotifier(Index: Integer);
    class function Save(ChangeName, ForceSave: Boolean): Boolean;

    { ProjectGroup }
    class procedure AddNewProject;
    class procedure AddExistingProject;
    class procedure RemoveProject(const AProject: IOTAProject);

    { SourceEditor }
    class function GetSourceEditor: IOTASourceEditor;
    class function AddSourceNotifier(const ANotifier: IOTANotifier): Integer;
    class function MarkSourceModified: Boolean;
    class procedure RemoveSourceNotifier(Index: Integer);
    class procedure SourceEditorShow;
    class function CreateReader: IOTAEditReader; { the names "CreateReader" and "CreateWriter" seems }
    class function CreateWriter: IOTAEditWriter; { to be misleading, also the comments in ToolsAPI.pas}
    class function CreateUndoableWriter: IOTAEditWriter;
    class function GetLinesInBuffer: Longint;
    class function SetSyntaxHighlighter(SyntaxHighlighter: TOTASyntaxHighlighter): TOTASyntaxHighlighter;

    { FormEditor }
    class function GetFormEditor: IOTAFormEditor;
    class function AddFormNotifier(const ANotifier: IOTANotifier): Integer;
    class function MarkFormModified: Boolean;
    class procedure RemoveFormNotifier(Index: Integer);
    class procedure FormEditorShow;
    class function GetRootComponent: IOTAComponent;
    class function FindComponent(const Name: string): IOTAComponent;
    class function GetComponentFromHandle(ComponentHandle: TOTAHandle): IOTAComponent;
    class function GetSelCount: Integer;
    class function GetSelComponent(Index: Integer): IOTAComponent;
    class function GetCreateParent: IOTAComponent;
    class function CreateComponent(const Container: IOTAComponent;
      const TypeName: string; X, Y, W, H: Integer): IOTAComponent; overload;
    class procedure GetFormResource(const Stream: IStream);

    { NTAFormEditor }
    class function GetNTAFormEditor: INTAFormEditor;
    class procedure GetNTAFormResource(Stream: TStream);


    { Current FormDesigner }
    class function CreateMethod(const Name: string; TypeData: PTypeData): TMethod;
    class function GetMethodName(const Method: TMethod): string;
    class procedure GetMethods(TypeData: PTypeData; Proc: TGetStrProc);
    class function GetPrivateDirectory: string;
    class procedure GetSelections(const List: IDesignerSelections);
    class function MethodExists(const Name: string): Boolean;
    class procedure RenameMethod(const CurName, NewName: string);
    class procedure SelectComponent(Instance: TPersistent);
    class procedure SetSelections(const List: IDesignerSelections);
    class procedure ShowMethod(const Name: string);
    class function UniqueName(const BaseName: string): string;
    class procedure GetComponentNames(TypeData: PTypeData; Proc: TGetStrProc);
    class function GetComponent(const Name: string): TComponent;
    class function GetNameOfComponent(Component: TComponent): string;
    class function GetObject(const Name: string): TPersistent;
    class function GetObjectName(Instance: TPersistent): string;
    class procedure GetObjectNames(TypeData: PTypeData; Proc: TGetStrProc);
    class function MethodFromAncestor(const Method: TMethod): Boolean;
    class function CreateComponent(ComponentClass: TComponentClass; Parent: TComponent;
      Left, Top, Width, Height: Integer): TComponent; overload;
    class function IsComponentLinkable(Component: TComponent): Boolean;
    class procedure MakeComponentLinkable(Component: TComponent);
    class function GetRoot: TComponent;
    class procedure Revert(Instance: TPersistent; PropInfo: PPropInfo);
    class function HasInterface: Boolean;
    class function HasInterfaceMember(const Name: string): Boolean;
    class procedure AddToInterface(InvKind: Integer; const Name: string; VT: Word;
      const TypeInfo: string); overload;
    class procedure GetProjectModules(Proc: TGetModuleProc);
    class function IsSourceReadOnly: Boolean;

    { Current Project }
    class function GetProjectModuleCount: Integer;
    class function GetModuleInfo(Index: Integer): IOTAModuleInfo;

    { Current ProjectBuilder }
    class function BuildProject(CompileMode: TOTACompileMode; Wait: Boolean): Boolean;

    { Current Process }
    class function AddNotifier(const Notifier: IOTAProcessNotifier): Integer; overload;
    class procedure Pause;
    class function ReadProcessMemory(Address: LongWord; Count: Integer; var Buffer): Integer;
    class procedure RemoveProcessNotifier(Index: Integer);
    class procedure Run(RunMode: TOTARunMode);
    class procedure Terminate;
    class function WriteProcessMemory(Address: LongWord; Count: Integer; var Buffer): Integer;

    { Current Thread }
    class function AddNotifier(const Notifier: IOTAThreadNotifier): Integer; overload;
    class function Evaluate(const ExprStr: string; ResultStr: PChar; ResultStrSize: LongWord;
      out CanModify: Boolean; AllowSideEffects: Boolean; FormatSpecifiers: PChar;
      out ResultAddr: LongWord; out ResultSize, ResultVal: LongWord): TOTAEvaluateResult;
    class function Modify(const ValueStr: string; ResultStr: PChar; ResultSize: LongWord;
      out ResultVal: Integer): TOTAEvaluateResult;
    class procedure GetCallPos(Index: Integer; out FileName: string; out LineNum: Integer);
    class function GetOSThreadID: LongWord;
    class procedure RemoveThreadNotifier(Index: Integer);

    { The current alternative file system, if one is associated }
    class function GetFileStream(const FileName: string; Mode: Integer): IStream;
    class function FileAge(const FileName: string): Longint;
    class function RenameFile(const OldName, NewName: string): Boolean;
    class function IsReadonly(const FileName: string): Boolean;
    class function IsFileBased: Boolean;
    class function DeleteFile(const FileName: string): Boolean;
    class function FileExists(const FileName: string): Boolean;
    class function GetTempFileName(const FileName: string): string;
    class function GetBackupFileName(const FileName: string): string;
    class function GetIDString: string;

    { The current EditReader }
    class function GetText(Position: Longint; Buffer: PChar; Count: Longint): Longint; overload;
    class procedure GetText(Stream: TMemoryStream); overload;
    class procedure GetText(List: TStringList); overload;
    class function GetTextLen: LongInt;

    { The current EditWriter }
    class procedure CopyTo(Pos: Longint);
    class procedure DeleteTo(Pos: Longint);
    class procedure Insert(Text: PChar);
    class function Position: Longint;
    class procedure ReplaceText(NewText: PChar);

    { The EditView at index 0 of the current editor }
    class function PosToCharPos(Pos: Longint): TOTACharPos;
    class function CharPosToPos(CharPos: TOTACharPos): Longint;
    class procedure ConvertPos(CharPosToEdPos: Boolean; var EditPos: TOTAEditPos;
      var CharPos: TOTACharPos);
    class procedure GetAttributeAtPos(const EdPos: TOTAEditPos; IncludeMargin: Boolean;
      var Element, LineFlag: Integer);
    class function SameView(EditView: IOTAEditView): Boolean;

    { The MainMenu }
    class function DispatchCommand(ACommand: Word): Boolean;
    class function DispatchPopup(AHandle: HMENU): Boolean;
    class function FindMenuItem(Value: Integer; Kind: TFindItemKind): TMenuItem; overload;
    class function FindMenuItem(Name: string): TMenuItem; overload;
    class procedure GetOle2AcceleratorTable(var AccelTable: HAccel;
      var AccelCount: Integer; Groups: array of Integer);
    class function GetMenuImages: TCustomImageList;
    class function GetMenuItems: TMenuItem;
    class procedure MenuAdd(Item: TMenuItem);
    class procedure MenuRemove(Item: TMenuItem);
    class procedure MenuInsert(Index: Integer; Item: TMenuItem);
    class procedure MenuDelete(Index: Integer);
    class function MenuIndexOf(Item: TMenuItem): Integer;

    { Access to the internal objects of Delphi }
    class procedure ScanForComponents(Root: TComponent; Name: string;
      var ToFind: TComponent);
    class function GetIdeObjectByName(Name: string): TComponent;
    class function GetObjectByName(Root: TComponent; Name: string): TComponent;

    { Access to the editor object of the first EditWindow }
    class function GetEditor: TComponent;
    class function GetCaretX: Integer;
    class function GetCaretY: Integer;
    class function GetCharXIndex: Integer;
    class function GetLineText: string;
    class function GetWordAtCaret: string;
    class function GetHotlinkText: string;
    class function GetLineCount: Integer;
    class function GetLinesInWindow: Integer;
    class function GetSearchAfter: Integer;
    class function GetSearchBegin: Integer;
    class function GetTopLine: Integer;

    { The new ToolServices }
    property ActionServices: IOTAActionServices read GetActionServices;
    property DebuggerServices: IOTADebuggerServices read GetDebuggerServices;
    property MessageServices: IOTAMessageServices read GetMessageServices;
    property ModuleServices: IOTAModuleServices read GetModuleServices;
    property PackageServices: IOTAPackageServices read GetPackageServices;
    property NTAServices: INTAServices read GetNTAServices;
    property Services: IOTAServices read GetServices;
    property WizardServices: IOTAWizardServices read GetWizardServices;

    { IOTAModuleServices }
    property CurrentModule: IOTAModule read GetCurrentModule;
    property ModuleCount: Integer read GetModuleCount;
    property Modules[Index: Integer]: IOTAModule read GetModule;

    { IOTADebuggerServices }
    property AddressBkptCount: Integer read GetAddressBkptCount;
    property AddressBkpts[Index: Integer]: IOTAAddressBreakpoint read GetAddressBkpt;
    property CurrentProcess: IOTAProcess read GetCurrentProcess write SetCurrentProcess;
    property ProcessCount: Integer read GetProcessCount;
    property Processes[Index: Integer]: IOTAProcess read GetProcess;
    property SourceBkptCount: Integer read GetSourceBkptCount;
    property SourceBkpts[Index: Integer]: IOTASourceBreakpoint read GetSourceBkpt;

    { IOTAPackageServices }
    property PackageCount: Integer read GetPackageCount;
    property PackageNames[Index: Integer]: string read GetPackageName;
    property ComponentCount[PkgIndex: Integer]: Integer read GetComponentCount;
    property ComponentNames[PkgIndex, CompIndex: Integer]: string read GetComponentName;

    { INTAServices }
    property ActionList: TCustomActionList read GetActionList;
    property ImageList: TCustomImageList read GetImageList;
    property MainMenu: TMainMenu read GetMainMenu;
    property ToolBar[const ToolBarName: string]: TToolBar read GetToolBar;

    { Current IOTAModule }
    property OwnerCount: Integer read GetOwnerCount;
    property Owners[Index: Integer]: IOTAProject read GetOwner;
    property FileName: string read GetFileName write SetFileName;
    property FileSystem: string read GetFileSystem write SetFileSystem;

    { ProjectGroup }
    property ProjectGroup: IOTAProjectGroup read GetProjectGroup;
    property ActiveProject: IOTAProject read GetActiveProject write SetActiveProject;
    property ProjectCount: Integer read GetProjectCount;
    property Projects[Index: Integer]: IOTAProject read GetProject;

    { SourceEditor }
    property SourceFileName: string read GetSourceFileName;
    property SourceModified: Boolean read GetSourceModified;
    property SourceModule: IOTAModule read GetSourceModule;
    property BlockStart: TOTACharPos read GetBlockStart write SetBlockStart;
    property BlockAfter: TOTACharPos read GetBlockAfter write SetBlockAfter;
    property BlockType: TOTABlockType read GetBlockType write SetBlockType;
    property BlockVisible: Boolean read GetBlockVisible write SetBlockVisible;
    property EditViewCount: Integer read GetEditViewCount;
    property EditViews[Index: Integer]: IOTAEditView read GetEditView;

    { FormEditor }
    property FormFileName: string read GetFormFileName;
    property FormModified: Boolean read GetFormModified;
    property FormModule: IOTAModule read GetFormModule;

    { NTAFormEditor }
    property FormDesigner: IFormDesigner read GetFormDesigner;

    { Current FormDesigner }
    property AncestorDesigner: IFormDesigner read GetAncestorDesigner;
    property IsDormant: Boolean read GetIsDormant;

    { Current Project }
    property ProjectOptions: IOTAProjectOptions read GetProjectOptions;
    property ProjectBuilder: IOTAProjectBuilder read GetProjectBuilder;

    { Current ProjectBuilder }
    property ShouldBuild: Boolean read GetShouldBuild;

     { Current Process }
    property CurrentThread: IOTAThread read GetCurrentThread write SetCurrentThread;
    property ProcessId: LongWord read GetProcessId;
    property ThreadCount: Integer read GetThreadCount;
    property Threads[Index: Integer]: IOTAThread read GetThread;

    { Current Thread }
    property CallCount: Integer read GetCallCount;
    property CallHeaders[Index: Integer]: string read GetCallHeader;
    property CurrentFile: string read GetCurrentFile;
    property CurrentLine: LongWord read GetCurrentLine;
    property Context: TContext read GetContext;
    property Handle: THandle read GetHandle;
    property State: TOTAThreadState read GetState;

    { The current EditWriter }
    property CurrentPos: TOTACharPos read GetCurrentPos;

    { The EditView at index 0 of the current editor }
    property CursorPos: TOTAEditPos read GetCursorPos write SetCursorPos;
    property TopPos: TOTAEditPos read GetTopPos write SetTopPos;
    property ViewSize: TSize read GetViewSize;

  published
    { Published-Deklarationen }
  end;


implementation

{ TOTA }

class function TOTA.GetActionServices: IOTAActionServices;
begin
  Result := BorlandIDEServices as IOTAActionServices;
end;

class procedure TOTA.AddCustomMessage(const CustomMsg: IOTACustomMessage);
begin
  GetMessageServices.AddCustomMessage(CustomMsg);
end;

class function TOTA.AddFileSystem(FileSystem: IOTAFileSystem): Integer;
begin
  Result := GetModuleServices.AddFileSystem(FileSystem);
end;

class function TOTA.AddNotifier(const Notifier: IOTADebuggerNotifier): Integer;
begin
  Result := GetDebuggerServices.AddNotifier(Notifier);
end;

class function TOTA.AddNotifier(const Notifier: IOTAIDENotifier): Integer;
begin
  Result := GetServices.AddNotifier(Notifier);
end;

class procedure TOTA.AddTitleMessage(const MessageStr: string);
begin
  GetMessageServices.AddTitleMessage(MessageStr);
end;

class procedure TOTA.AddToolMessage(const FileName, MessageStr,
  PrefixStr: string; LineNumber, ColumnNumber: Integer);
begin
  GetMessageServices.AddToolMessage(FileName, MessageStr, PrefixStr, LineNumber, ColumnNumber);
end;

class function TOTA.AddWizard(const AWizard: IOTAWizard): Integer;
begin
  Result := GetWizardServices.AddWizard(AWizard);
end;

class procedure TOTA.AttachProcess(Pid: Integer; const RemoteHost: string);
begin
  GetDebuggerServices.AttachProcess(Pid, RemoteHost);
end;

class procedure TOTA.ClearAllMessages;
begin
  GetMessageServices.ClearAllMessages;
end;

class procedure TOTA.ClearCompilerMessages;
begin
  GetMessageServices.ClearCompilerMessages;
end;

class procedure TOTA.ClearSearchMessages;
begin
  GetMessageServices.ClearSearchMessages;
end;

class procedure TOTA.ClearToolMessages;
begin
  GetMessageServices.ClearToolMessages;
end;

class function TOTA.CloseAll: Boolean;
begin
  Result := GetModuleServices.CloseAll;
end;

class function TOTA.CloseFile(const FileName: string): Boolean;
begin
  Result := GetActionServices.CloseFile(FileName);
end;

class function TOTA.CreateModule(const Creator: IOTACreator): IOTAModule;
begin
  Result := GetModuleServices.CreateModule(Creator);
end;

class procedure TOTA.CreateProcess(const ExeName, Args, RemoteHost: string);
begin
  GetDebuggerServices.CreateProcess(ExeName, Args, RemoteHost);
end;

class function TOTA.GetCurrentModule: IOTAModule;
begin
  Result := GetModuleServices.CurrentModule;
  if Result = nil then raise exception.Create('Sorry, no Module available');
end;

class function TOTA.GetDebuggerServices: IOTADebuggerServices;
begin
  Result := BorlandIDEServices as IOTADebuggerServices;
end;

class procedure TOTA.EnumerateRunningProcesses(
  Callback: TEnumerateProcessesCallback; Param: Pointer;
  const HostName: string);
begin
  GetDebuggerServices.EnumerateRunningProcesses(Callback, Param, HostName);
end;

class function TOTA.FindFileSystem(const Name: string): IOTAFileSystem;
begin
  Result := GetModuleServices.FindFileSystem(Name);
  if Result = nil then raise exception.Create('Sorry, the desired file system does not exist');
end;

class function TOTA.FindFormModule(const FormName: string): IOTAModule;
begin
  Result := GetModuleServices.FindFormModule(FormName);
end;

class function TOTA.FindModule(const FileName: string): IOTAModule;
begin
  Result := GetModuleServices.FindModule(FileName);
end;

class function TOTA.GetAddressBkpt(Index: Integer): IOTAAddressBreakpoint;
begin
  Result := GetDebuggerServices.GetAddressBkpt(Index);
end;

class function TOTA.GetAddressBkptCount: Integer;
begin
  Result := GetDebuggerServices.GetAddressBkptCount;
end;

class function TOTA.GetBaseRegistryKey: string;
begin
  Result := GetServices.GetBaseRegistryKey;
end;

class function TOTA.GetComponentCount(PkgIndex: Integer): Integer;
begin
  Result := GetPackageServices.GetComponentCount(PkgIndex);
end;

class function TOTA.GetComponentName(PkgIndex, CompIndex: Integer): string;
begin
  Result := GetPackageServices.GetComponentName(PkgIndex, CompIndex);
end;

class function TOTA.GetCurrentProcess: IOTAProcess;
begin
  Result := GetDebuggerServices.GetCurrentProcess;
  if Result = nil then raise exception.Create('Sorry, no process available');
end;

class function TOTA.GetEnvironmentOptions: IOTAEnvironmentOptions;
begin
  Result := GetServices.GetEnvironmentOptions;
end;

class function TOTA.GetModule(Index: Integer): IOTAModule;
begin
  Result := GetModuleServices.GetModule(Index);
end;

class function TOTA.GetModuleCount: Integer;
begin
  Result := GetModuleServices.GetModuleCount;
end;

class procedure TOTA.GetNewModuleAndClassName(const Prefix: string;
  var UnitIdent, Name, FileName: string);
begin
  GetModuleServices.GetNewModuleAndClassName(Prefix, UnitIdent, Name, FileName);
end;

class function TOTA.GetPackageCount: Integer;
begin
  Result := GetPackageServices.GetPackageCount;
end;

class function TOTA.GetPackageName(Index: Integer): string;
begin
  Result := GetPackageServices.GetPackageName(Index);
end;

class function TOTA.GetParentHandle: HWND;
begin
  Result := GetServices.GetParentHandle;
end;

class function TOTA.GetProcess(Index: Integer): IOTAProcess;
begin
  Result := GetDebuggerServices.GetProcess(Index);
end;

class function TOTA.GetProcessCount: Integer;
begin
  Result := GetDebuggerServices.GetProcessCount;
end;

class function TOTA.GetProductIdentifier: string;
begin
  Result := GetServices.GetProductIdentifier;
end;

class function TOTA.GetSourceBkpt(Index: Integer): IOTASourceBreakpoint;
begin
  Result := GetDebuggerServices.GetSourceBkpt(Index);
end;

class function TOTA.GetSourceBkptCount: Integer;
begin
  Result := GetDebuggerServices.GetSourceBkptCount;
end;

class procedure TOTA.LogString(const LogStr: string);
begin
  GetDebuggerServices.LogString(LogStr);
end;

class function TOTA.GetMessageServices: IOTAMessageServices;
begin
  Result := BorlandIDEServices as IOTAMessageServices;
end;

class function TOTA.GetModuleServices: IOTAModuleServices;
begin
  Result := BorlandIDEServices as IOTAModuleServices;
end;

class function TOTA.NewAddressBreakpoint(Address, Length: LongWord;
  AccessType: TOTAAccessType; AProcess: IOTAProcess): IOTABreakpoint;
begin
  Result := GetDebuggerServices.NewAddressBreakpoint(Address, Length, AccessType, AProcess);
end;

class function TOTA.NewModule: Boolean;
begin
  Result := GetModuleServices.NewModule;
end;

class function TOTA.NewModuleBreakpoint(const ModuleName: string;
  AProcess: IOTAProcess): IOTABreakpoint;
begin
  Result := GetDebuggerServices.NewModuleBreakpoint(ModuleName, AProcess);
end;

class function TOTA.NewSourceBreakpoint(const FileName: string;
  LineNumber: Integer; AProcess: IOTAProcess): IOTABreakpoint;
begin
  Result := GetDebuggerServices.NewSourceBreakpoint(FileName, LineNumber, AProcess);
end;

class function TOTA.OpenFile(const FileName: string): Boolean;
begin
  Result := GetActionServices.OpenFile(FileName);
end;

class function TOTA.OpenProject(const ProjName: string;
  NewProjGroup: Boolean): Boolean;
begin
  Result := GetActionServices.OpenProject(ProjName, NewProjGroup);
end;

class function TOTA.GetPackageServices: IOTAPackageServices;
begin
  Result := BorlandIDEServices as IOTAPackageServices;
end;

class function TOTA.ReloadFile(const FileName: string): Boolean;
begin
  Result := GetActionServices.ReloadFile(FileName);
end;

class procedure TOTA.RemoveFileSystem(Index: Integer);
begin
  GetModuleServices.RemoveFileSystem(Index);
end;

class procedure TOTA.RemoveDebuggerNotifier(Index: Integer);
begin
  GetDebuggerServices.RemoveNotifier(Index);
end;

class procedure TOTA.RemoveIDENotifier(Index: Integer);
begin
  GetServices.RemoveNotifier(Index);
end;

class procedure TOTA.RemoveWizard(Index: Integer);
begin
  GetWizardServices.RemoveWizard(Index);
end;

class function TOTA.SaveAll: Boolean;
begin
  Result := GetModuleServices.SaveAll;
end;

class function TOTA.SaveFile(const FileName: string): Boolean;
begin
  Result := GetActionServices.SaveFile(FileName);
end;

class function TOTA.GetNTAServices: INTAServices;
begin
  Result := BorlandIDEServices as INTAServices;
end;

class function TOTA.GetServices: IOTAServices;
begin
  Result := BorlandIDEServices as IOTAServices;
end;

class procedure TOTA.SetCurrentProcess(Process: IOTAProcess);
begin
  GetDebuggerServices.SetCurrentProcess(Process);
end;

class function TOTA.GetWizardServices: IOTAWizardServices;
begin
  Result := BorlandIDEServices as IOTAWizardServices;
end;

class function TOTA.AddNotifier(const ANotifier: IOTAModuleNotifier): Integer;
begin
  Result := GetCurrentModule.AddNotifier(ANotifier);
end;

class procedure TOTA.AddToInterface;
begin
  GetCurrentModule.AddToInterface;
end;

class function TOTA.Close: Boolean;
begin
  Result := GetCurrentModule.Close;
end;

class function TOTA.GetFileName: string;
begin
  Result := GetCurrentModule.GetFileName;
end;

class function TOTA.GetFileSystem: string;
begin
  Result := GetCurrentModule.GetFileSystem;
  if Result = '' then raise exception.create('Sorry, no alternative FileSystem is associated');
end;

class function TOTA.GetModuleFileCount: Integer;
begin
  Result := GetCurrentModule.GetModuleFileCount;
end;

class function TOTA.GetModuleFileEditor(Index: Integer): IOTAEditor;
begin
  if GetCurrentModule.GetModuleFileCount = 0 then raise exception.Create('Sorry, no file module available');
  Result := GetCurrentModule.GetModuleFileEditor(Index);
end;

class function TOTA.GetOwner(Index: Integer): IOTAProject;
begin
  Result := GetCurrentModule.GetOwner(Index);
end;

class function TOTA.GetOwnerCount: Integer;
begin
  Result := GetCurrentModule.GetOwnerCount;
end;

class function TOTA.HasCoClasses: Boolean;
begin
  Result := GetCurrentModule.HasCoClasses;
end;

class procedure TOTA.RemoveModuleNotifier(Index: Integer);
begin
  GetCurrentModule.RemoveNotifier(Index);
end;

class function TOTA.Save(ChangeName, ForceSave: Boolean): Boolean;
begin
  Result := GetCurrentModule.Save(ChangeName, ForceSave);
end;

class procedure TOTA.SetFileName(const AFileName: string);
begin
  GetCurrentModule.SetFileName(AFileName);
end;

class procedure TOTA.SetFileSystem(const AFileSystem: string);
begin
  GetCurrentModule.SetFileSystem(AFileSystem);
end;

class function TOTA.GetProjectGroup: IOTAProjectGroup;
var
  AModuleServices: IOTAModuleServices;
  AModule: IOTAModule;
  i: integer;
  AProjectGroup: IOTAProjectGroup;
begin
  AModuleServices := BorlandIDEServices as IOTAModuleServices;
  for i := 0 to AModuleServices.ModuleCount - 1 do
  begin
    AModule := AModuleServices.Modules[i];
    if AModule.QueryInterface(IOTAProjectGroup, AProjectGroup) = S_OK then Break;
  end;
  Result := AProjectGroup;
  AModuleServices := nil;
  AModule := nil;
  AProjectGroup := nil;
  if Result = nil then raise exception.Create('Sorry, no ProjectGroup available');
end;

class procedure TOTA.AddExistingProject;
begin
  GetProjectGroup.AddExistingProject;
end;

class procedure TOTA.AddNewProject;
begin
  GetProjectGroup.AddNewProject;
end;

class function TOTA.GetActiveProject: IOTAProject;
begin
  Result := GetProjectGroup.GetActiveProject;
  if Result = nil then raise exception.Create('Sorry, no Project available');
end;

class function TOTA.GetProject(Index: Integer): IOTAProject;
begin
  Result := GetProjectGroup.GetProject(Index);
end;

class function TOTA.GetProjectCount: Integer;
begin
  Result := GetProjectGroup.GetProjectCount;
end;

class procedure TOTA.RemoveProject(const AProject: IOTAProject);
begin
  GetProjectGroup.RemoveProject(AProject);
end;

class procedure TOTA.SetActiveProject(const AProject: IOTAProject);
begin
  GetProjectGroup.SetActiveProject(AProject);
end;

class function TOTA.AddMasked(Image: TBitmap; MaskColor: TColor): Integer;
begin
  Result := GetNTAServices.AddMasked(Image, MaskColor);
end;

class function TOTA.GetActionList: TCustomActionList;
begin
  Result := GetNTAServices.GetActionList;
end;

class function TOTA.GetImageList: TCustomImageList;
begin
  Result := GetNTAServices.GetImageList;
end;

class function TOTA.GetMainMenu: TMainMenu;
begin
  Result := GetNTAServices.GetMainMenu;
end;

class function TOTA.GetToolBar(const ToolBarName: string): TToolBar;
begin
  Result := GetNTAServices.GetToolBar(ToolBarName);
end;

class function TOTA.GetSourceEditor: IOTASourceEditor;
var
  AModule: IOTAModule;
  i: integer;
  AEditor: IOTAEditor;
  ASourceEditor: IOTASourceEditor;
begin
  ASourceEditor := nil;
  AModule := GetCurrentModule;
  if AModule.GetModuleFileCount = 0 then raise exception.Create('Sorry, no file module available');
  for i := 0 to AModule.GetModuleFileCount - 1 do
  begin
    AEditor := AModule.GetModuleFileEditor(I);
    if AEditor = nil then raise exception.Create('Sorry, no ModuleFileEditor available');
    if AEditor.QueryInterface(IOTASourceEditor, ASourceEditor) = S_OK then Break;
  end;
  Result := ASourceEditor;
  AModule := nil;
  AEditor := nil;
  ASourceEditor := nil;
  if Result = nil then raise exception.Create('Sorry, no SourceEditor available');
end;

class function TOTA.GetSourceFileName: string;
begin
  Result := GetSourceEditor.FileName;
end;

class function TOTA.AddSourceNotifier(const ANotifier: IOTANotifier): Integer;
begin
  Result := GetSourceEditor.AddNotifier(ANotifier);
end;

class function TOTA.GetSourceModified: Boolean;
begin
  Result := GetSourceEditor.Modified;
end;

class function TOTA.GetSourceModule: IOTAModule;
begin
  Result := GetSourceEditor.Module;
end;

class function TOTA.MarkSourceModified: Boolean;
begin
  Result := GetSourceEditor.MarkModified;
end;

class procedure TOTA.RemoveSourceNotifier(Index: Integer);
begin
  GetSourceEditor.RemoveNotifier(Index);
end;

class procedure TOTA.SourceEditorShow;
begin
  GetSourceEditor.Show;
end;

class function TOTA.CreateReader: IOTAEditReader;
begin
  Result := GetSourceEditor.CreateReader;
  if Result = nil then raise exception.Create('Sorry, creation of reader failed');
end;

class function TOTA.CreateUndoableWriter: IOTAEditWriter;
begin
  Result := GetSourceEditor.CreateUndoableWriter;
  if Result = nil then raise exception.Create('Sorry, creation of undoable writer failed');
end;

class function TOTA.CreateWriter: IOTAEditWriter;
begin
  Result := GetSourceEditor.CreateWriter;
  if Result = nil then raise exception.Create('Sorry, creation of writer failed');
end;

class function TOTA.GetBlockAfter: TOTACharPos;
begin
  Result := GetSourceEditor.GetBlockAfter;
end;

class function TOTA.GetBlockStart: TOTACharPos;
begin
  Result := GetSourceEditor.GetBlockStart;
end;

class function TOTA.GetBlockType: TOTABlockType;
begin
  Result := GetSourceEditor.GetBlockType;
end;

class function TOTA.GetBlockVisible: Boolean;
begin
  Result := GetSourceEditor.GetBlockVisible;
end;

class function TOTA.GetEditView(Index: Integer): IOTAEditView;
begin
  if GetEditViewCount = 0 then raise exception.Create('Sorry, no EditView available');
  Result := GetSourceEditor.GetEditView(Index);
end;

class function TOTA.GetEditViewCount: Integer;
begin
  Result := GetSourceEditor.GetEditViewCount;
end;

class function TOTA.GetLinesInBuffer: Longint;
begin
  Result := GetSourceEditor.GetLinesInBuffer;
end;

class procedure TOTA.SetBlockAfter(const Value: TOTACharPos);
begin
  GetSourceEditor.SetBlockAfter(Value);
end;

class procedure TOTA.SetBlockStart(const Value: TOTACharPos);
begin
  GetSourceEditor.SetBlockStart(Value);
end;

class procedure TOTA.SetBlockType(Value: TOTABlockType);
begin
  GetSourceEditor.SetBlockType(Value);
end;

class procedure TOTA.SetBlockVisible(Value: Boolean);
begin
  GetSourceEditor.SetBlockVisible(Value);
end;

class function TOTA.SetSyntaxHighlighter(
  SyntaxHighlighter: TOTASyntaxHighlighter): TOTASyntaxHighlighter;
begin
  Result := GetSourceEditor.SetSyntaxHighlighter(SyntaxHighlighter);
end;

class function TOTA.GetFormEditor: IOTAFormEditor;
var
  AModule: IOTAModule;
  i: integer;
  AEditor: IOTAEditor;
  AFormEditor: IOTAFormEditor;
begin
  AFormEditor := nil;
  AModule := GetCurrentModule;
  for i := 0 to AModule.GetModuleFileCount - 1 do
  begin
    AEditor := AModule.GetModuleFileEditor(I);
    if AEditor = nil then raise exception.Create('Sorry, no ModuleFileEditor available');
    if AEditor.QueryInterface(IOTAFormEditor, AFormEditor) = S_OK then Break;
  end;
  Result := AFormEditor;
  AModule := nil;
  AEditor := nil;
  AFormEditor := nil;
  if Result = nil then raise exception.Create('Sorry, no FormEditor available');
end;

class function TOTA.GetFormFileName: string;
begin
  Result := GetFormEditor.FileName;
end;

class function TOTA.AddFormNotifier(const ANotifier: IOTANotifier): Integer;
begin
  Result := GetFormEditor.AddNotifier(ANotifier);
end;

class procedure TOTA.FormEditorShow;
begin
  GetFormEditor.Show;
end;

class function TOTA.GetFormModified: Boolean;
begin
  Result := GetFormEditor.Modified;
end;

class function TOTA.GetFormModule: IOTAModule;
begin
  Result := GetFormEditor.Module;
end;

class function TOTA.MarkFormModified: Boolean;
begin
  Result := GetFormEditor.MarkModified;
end;

class procedure TOTA.RemoveFormNotifier(Index: Integer);
begin
  GetFormEditor.RemoveNotifier(Index);
end;

class function TOTA.CreateComponent(const Container: IOTAComponent;
  const TypeName: string; X, Y, W, H: Integer): IOTAComponent;
begin
  Result := GetFormEditor.CreateComponent(Container, TypeName, X, Y, W, H);
end;

class function TOTA.FindComponent(const Name: string): IOTAComponent;
begin
  Result := GetFormEditor.FindComponent(Name);
end;

class function TOTA.GetComponentFromHandle(
  ComponentHandle: TOTAHandle): IOTAComponent;
begin
  Result := GetFormEditor.GetComponentFromHandle(ComponentHandle);
end;

class function TOTA.GetCreateParent: IOTAComponent;
begin
  Result := GetFormEditor.GetCreateParent;
end;

class procedure TOTA.GetFormResource(const Stream: IStream);
begin
  GetFormEditor.GetFormResource(Stream);
end;

class function TOTA.GetRootComponent: IOTAComponent;
begin
  Result := GetFormEditor.GetRootComponent;
end;

class function TOTA.GetSelComponent(Index: Integer): IOTAComponent;
begin
  Result := GetFormEditor.GetSelComponent(Index);
end;

class function TOTA.GetSelCount: Integer;
begin
  Result := GetFormEditor.GetSelCount;
end;

class function TOTA.GetProjectBuilder: IOTAProjectBuilder;
begin
  Result := GetActiveProject.GetProjectBuilder;
end;

class function TOTA.GetModuleInfo(Index: Integer): IOTAModuleInfo;
begin
  Result := GetActiveProject.GetModule(Index);
end;

class function TOTA.GetProjectModuleCount: Integer;
begin
  Result := GetActiveProject.GetModuleCount;
end;

class function TOTA.GetProjectOptions: IOTAProjectOptions;
begin
  Result := GetActiveProject.GetProjectOptions;
end;

class function TOTA.BuildProject(CompileMode: TOTACompileMode;
  Wait: Boolean): Boolean;
begin
  Result := GetProjectBuilder.BuildProject(CompileMode, Wait);
end;

class function TOTA.GetShouldBuild: Boolean;
begin
  Result := GetProjectBuilder.ShouldBuild;
end;

class function TOTA.AddNotifier(const Notifier: IOTAProcessNotifier): Integer;
begin
  Result := GetCurrentProcess.AddNotifier(Notifier);
end;

class function TOTA.GetCurrentThread: IOTAThread;
begin
  Result := GetCurrentProcess.GetCurrentThread;
  if Result = nil then raise exception.Create('Sorry, no thread available');
end;

class function TOTA.GetProcessId: LongWord;
begin
  Result := GetCurrentProcess.GetProcessId;
end;

class function TOTA.GetThread(Index: Integer): IOTAThread;
begin
  Result := GetCurrentProcess.GetThread(Index);
end;

class function TOTA.GetThreadCount: Integer;
begin
  Result := GetCurrentProcess.GetThreadCount;
end;

class procedure TOTA.Pause;
begin
  GetCurrentProcess.Pause;
end;

class function TOTA.ReadProcessMemory(Address: LongWord; Count: Integer;
  var Buffer): Integer;
begin
  Result := GetCurrentProcess.ReadProcessMemory(Address, Count, Buffer);
end;

class procedure TOTA.Run(RunMode: TOTARunMode);
begin
  GetCurrentProcess.Run(RunMode);
end;

class procedure TOTA.SetCurrentThread(Value: IOTAThread);
begin
  GetCurrentProcess.SetCurrentThread(Value);
end;

class procedure TOTA.Terminate;
begin
  GetCurrentProcess.Terminate;
end;

class function TOTA.WriteProcessMemory(Address: LongWord; Count: Integer;
  var Buffer): Integer;
begin
  Result := GetCurrentProcess.WriteProcessMemory(Address, Count, Buffer);
end;

class procedure TOTA.RemoveProcessNotifier(Index: Integer);
begin
  GetCurrentProcess.RemoveNotifier(Index);
end;

class function TOTA.AddNotifier(const Notifier: IOTAThreadNotifier): Integer;
begin
  Result := GetCurrentThread.AddNotifier(Notifier);
end;

class function TOTA.Evaluate(const ExprStr: string; ResultStr: PChar;
  ResultStrSize: LongWord; out CanModify: Boolean;
  AllowSideEffects: Boolean; FormatSpecifiers: PChar; out ResultAddr,
  ResultSize, ResultVal: LongWord): TOTAEvaluateResult;
begin
  Result := GetCurrentThread.Evaluate(ExprStr, ResultStr, ResultStrSize,
    CanModify, AllowSideEffects, FormatSpecifiers, ResultAddr, ResultSize,
    ResultVal);
end;

class function TOTA.GetCallCount: Integer;
begin
  Result := GetCurrentThread.GetCallCount;
end;

class function TOTA.GetCallHeader(Index: Integer): string;
begin
  Result := GetCurrentThread.GetCallHeader(Index);
end;

class procedure TOTA.GetCallPos(Index: Integer; out FileName: string;
  out LineNum: Integer);
begin
  GetCurrentThread.GetCallPos(Index, FileName, LineNum);
end;

class function TOTA.GetContext: TContext;
begin
  Result := GetCurrentThread.GetContext;
end;

class function TOTA.GetCurrentFile: string;
begin
  Result := GetCurrentThread.GetCurrentFile;
end;

class function TOTA.GetCurrentLine: LongWord;
begin
  Result := GetCurrentThread.GetCurrentLine;
end;

class function TOTA.GetHandle: THandle;
begin
  Result := GetCurrentThread.GetHandle;
end;

class function TOTA.GetOSThreadID: LongWord;
begin
  Result := GetCurrentThread.GetOSThreadID;
end;

class function TOTA.GetState: TOTAThreadState;
begin
  Result := GetCurrentThread.GetState;
end;

class function TOTA.Modify(const ValueStr: string; ResultStr: PChar;
  ResultSize: LongWord; out ResultVal: Integer): TOTAEvaluateResult;
begin
  Result := GetCurrentThread.Modify(ValueStr, ResultStr, ResultSize, ResultVal);
end;

class procedure TOTA.RemoveThreadNotifier(Index: Integer);
begin
  GetCurrentThread.RemoveNotifier(Index);
end;

class function TOTA.DeleteFile(const FileName: string): Boolean;
begin
  Result := FindFileSystem(GetFileSystem).DeleteFile(FileName);
end;

class function TOTA.FileAge(const FileName: string): Longint;
begin
  Result := FindFileSystem(GetFileSystem).FileAge(FileName);
end;

class function TOTA.FileExists(const FileName: string): Boolean;
begin
  Result := FindFileSystem(GetFileSystem).FileExists(FileName);
end;

class function TOTA.GetBackupFileName(const FileName: string): string;
begin
  Result := FindFileSystem(GetFileSystem).GetBackupFileName(FileName);
end;

class function TOTA.GetFileStream(const FileName: string; Mode: Integer): IStream;
begin
  Result := FindFileSystem(GetFileSystem).GetFileStream(FileName, Mode);
end;

class function TOTA.GetIDString: string;
begin
  Result := FindFileSystem(GetFileSystem).GetIDString;
end;

class function TOTA.GetTempFileName(const FileName: string): string;
begin
  Result := FindFileSystem(GetFileSystem).GetTempFileName(FileName);
end;

class function TOTA.IsFileBased: Boolean;
begin
  Result := FindFileSystem(GetFileSystem).IsFileBased;
end;

class function TOTA.IsReadonly(const FileName: string): Boolean;
begin
  Result := FindFileSystem(GetFileSystem).IsReadonly(FileName);
end;

class function TOTA.RenameFile(const OldName, NewName: string): Boolean;
begin
  Result := FindFileSystem(GetFileSystem).RenameFile(OldName, NewName);
end;

class function TOTA.CharPosToPos(CharPos: TOTACharPos): Longint;
begin
  Result := GetEditView(0).CharPosToPos(CharPos);
end;

class procedure TOTA.ConvertPos(CharPosToEdPos: Boolean;
  var EditPos: TOTAEditPos; var CharPos: TOTACharPos);
begin
  GetEditView(0).ConvertPos(CharPosToEdPos, EditPos, CharPos);
end;

class procedure TOTA.GetAttributeAtPos(const EdPos: TOTAEditPos;
  IncludeMargin: Boolean; var Element, LineFlag: Integer);
begin
  GetEditView(0).GetAttributeAtPos(EdPos, IncludeMargin, Element, LineFlag);
end;

class function TOTA.GetCursorPos: TOTAEditPos;
begin
  Result := GetEditView(0).GetCursorPos;
end;

class function TOTA.GetTopPos: TOTAEditPos;
begin
  Result := GetEditView(0).GetTopPos;
end;

class function TOTA.GetViewSize: TSize;
begin
  Result := GetEditView(0).GetViewSize;
end;

class function TOTA.PosToCharPos(Pos: Integer): TOTACharPos;
begin
  Result := GetEditView(0).PosToCharPos(Pos);
end;

class function TOTA.SameView(EditView: IOTAEditView): Boolean;
begin
  Result := GetEditView(0).SameView(EditView);
end;

class procedure TOTA.SetCursorPos(const Value: TOTAEditPos);
begin
  GetEditView(0).SetCursorPos(Value);
end;

class procedure TOTA.SetTopPos(const Value: TOTAEditPos);
begin
  GetEditView(0).SetTopPos(Value);
end;

class function TOTA.GetFormDesigner: IFormDesigner;
begin
  Result := GetNTAFormEditor.GetFormDesigner;
  if Result = nil then raise exception.create('Sorry, no FormDesigner available');
end;

class procedure TOTA.GetNTAFormResource(Stream: TStream);
begin
  GetNTAFormEditor.GetFormResource(Stream);
end;

class function TOTA.GetNTAFormEditor: INTAFormEditor;
begin
  Result := GetFormEditor as INTAFormEditor;
end;

class procedure TOTA.AddToInterface(InvKind: Integer; const Name: string;
  VT: Word; const TypeInfo: string);
begin
  GetFormDesigner.AddToInterface(InvKind, Name, VT, TypeInfo);
end;

class function TOTA.CreateComponent(ComponentClass: TComponentClass;
  Parent: TComponent; Left, Top, Width, Height: Integer): TComponent;
begin
  Result := GetFormDesigner.CreateComponent(ComponentClass, Parent, Left, Top, Width, Height);
end;

class function TOTA.CreateMethod(const Name: string;
  TypeData: PTypeData): TMethod;
begin
  Result := GetFormDesigner.CreateMethod(Name, TypeData);
end;

class function TOTA.GetAncestorDesigner: IFormDesigner;
begin
  Result := GetFormDesigner.GetAncestorDesigner;
end;

class function TOTA.GetComponent(const Name: string): TComponent;
begin
  Result := GetFormDesigner.GetComponent(Name);
end;

class procedure TOTA.GetComponentNames(TypeData: PTypeData;
  Proc: TGetStrProc);
begin
  GetFormDesigner.GetComponentNames(TypeData, Proc);
end;

class function TOTA.GetIsDormant: Boolean;
begin
  Result := GetFormDesigner.GetIsDormant;
end;

class function TOTA.GetMethodName(const Method: TMethod): string;
begin
  Result := GetFormDesigner.GetMethodName(Method);
end;

class procedure TOTA.GetMethods(TypeData: PTypeData; Proc: TGetStrProc);
begin
  GetFormDesigner.GetMethods(TypeData, Proc);
end;

class function TOTA.GetNameOfComponent(Component: TComponent): string;
begin
  Result := GetFormDesigner.GetComponentName(Component);
end;

class function TOTA.GetObject(const Name: string): TPersistent;
begin
  Result := GetFormDesigner.GetObject(Name);
end;

class function TOTA.GetObjectName(Instance: TPersistent): string;
begin
  Result := GetFormDesigner.GetObjectName(Instance);
end;

class procedure TOTA.GetObjectNames(TypeData: PTypeData;
  Proc: TGetStrProc);
begin
  GetFormDesigner.GetObjectNames(TypeData, Proc);
end;

class function TOTA.GetPrivateDirectory: string;
begin
  Result := GetFormDesigner.GetPrivateDirectory;
end;

class procedure TOTA.GetProjectModules(Proc: TGetModuleProc);
begin
  GetFormDesigner.GetProjectModules(Proc);
end;

class function TOTA.GetRoot: TComponent;
begin
  Result := GetFormDesigner.GetRoot;
end;

class procedure TOTA.GetSelections(const List: IDesignerSelections);
begin
  GetFormDesigner.GetSelections(List);
end;

class function TOTA.HasInterface: Boolean;
begin
  Result := GetFormDesigner.HasInterface;
end;

class function TOTA.HasInterfaceMember(const Name: string): Boolean;
begin
  Result := GetFormDesigner.HasInterfaceMember(Name);
end;

class function TOTA.IsComponentLinkable(Component: TComponent): Boolean;
begin
  Result := GetFormDesigner.IsComponentLinkable(Component);
end;

class function TOTA.IsSourceReadOnly: Boolean;
begin
  Result := GetFormDesigner.IsSourceReadOnly;
end;

class procedure TOTA.MakeComponentLinkable(Component: TComponent);
begin
  GetFormDesigner.MakeComponentLinkable(Component);
end;

class function TOTA.MethodExists(const Name: string): Boolean;
begin
  Result := GetFormDesigner.MethodExists(Name);
end;

class function TOTA.MethodFromAncestor(const Method: TMethod): Boolean;
begin
  Result := GetFormDesigner.MethodFromAncestor(Method);
end;

class procedure TOTA.RenameMethod(const CurName, NewName: string);
begin
  GetFormDesigner.RenameMethod(CurName, NewName);
end;

class procedure TOTA.Revert(Instance: TPersistent; PropInfo: PPropInfo);
begin
  GetFormDesigner.Revert(Instance, PropInfo);
end;

class procedure TOTA.SelectComponent(Instance: TPersistent);
begin
  GetFormDesigner.SelectComponent(Instance);
end;

class procedure TOTA.SetSelections(const List: IDesignerSelections);
begin
  GetFormDesigner.SetSelections(List);
end;

class procedure TOTA.ShowMethod(const Name: string);
begin
  GetFormDesigner.ShowMethod(Name);
end;

class function TOTA.UniqueName(const BaseName: string): string;
begin
  Result := GetFormDesigner.UniqueName(BaseName);
end;

class function TOTA.DispatchCommand(ACommand: Word): Boolean;
begin
  Result := GetMainMenu.DispatchCommand(ACommand);
end;

class function TOTA.DispatchPopup(AHandle: HMENU): Boolean;
begin
  Result := GetMainMenu.DispatchPopup(AHandle);
end;

class function TOTA.FindMenuItem(Value: Integer;
  Kind: TFindItemKind): TMenuItem;
begin
  Result := GetMainMenu.FindItem(Value, Kind);
end;

class function TOTA.GetMenuImages: TCustomImageList;
begin
  Result := GetMainMenu.Images;
end;

class function TOTA.GetMenuItems: TMenuItem;
begin
  Result := GetMainMenu.Items;
end;

class procedure TOTA.GetOle2AcceleratorTable(var AccelTable: HAccel;
  var AccelCount: Integer; Groups: array of Integer);
begin
  GetMainMenu.GetOle2AcceleratorTable(AccelTable, AccelCount, Groups);
end;

class procedure TOTA.MenuAdd(Item: TMenuItem);
begin
  GetMenuItems.Add(Item);
end;

class procedure TOTA.MenuDelete(Index: Integer);
begin
  GetMenuItems.Delete(Index);
end;

class function TOTA.MenuIndexOf(Item: TMenuItem): Integer;
begin
  Result := GetMenuItems.IndexOf(Item);
end;

class procedure TOTA.MenuInsert(Index: Integer; Item: TMenuItem);
begin
  GetMenuItems.Insert(Index, Item);
end;

class procedure TOTA.MenuRemove(Item: TMenuItem);
begin
  GetMenuItems.Remove(Item);
end;

class function TOTA.FindMenuItem(Name: string): TMenuItem;
var
  I: Integer;
  aMenu: TMenu;
begin
  Result := nil;
  aMenu := GetMainMenu;
  for I := 0 to aMenu.Items.Count - 1 do
    if CompareText(aMenu.Items[I].Name, Name) = 0 then
    begin
      Result := aMenu.Items[I];
      break;
    end;
end;

class function TOTA.GetIdeObjectByName(Name: string): TComponent;
begin
  ScanForComponents(Application, Name, Result);
end;

class function TOTA.GetObjectByName(Root: TComponent;
  Name: string): TComponent;
begin
  ScanForComponents(Root, Name, Result);
end;

class procedure TOTA.ScanForComponents(Root: TComponent;
  Name: string; var ToFind: TComponent);
var
  I: Integer;
begin
  ToFind := Root.FindComponent(Name);
  for I := 0 to Root.ComponentCount - 1 do
  begin
    if ToFind <> nil then break;
    ScanForComponents(Root.Components[I], Name, ToFind);
  end;
end;

class procedure TOTA.CopyTo(Pos: Integer);
begin
  CreateWriter.CopyTo(Pos);
end;

class procedure TOTA.DeleteTo(Pos: Integer);
begin
  CreateWriter.DeleteTo(Pos);
end;

class function TOTA.GetCurrentPos: TOTACharPos;
begin
  Result := CreateWriter.GetCurrentPos;
end;

class function TOTA.GetText(Position: Integer; Buffer: PChar;
  Count: Integer): Longint;
begin
  Result := CreateReader.GetText(Position, Buffer, Count);
end;

class procedure TOTA.Insert(Text: PChar);
begin
  CreateWriter.Insert(Text);
end;

class function TOTA.Position: Longint;
begin
  Result := CreateWriter.Position;
end;

class procedure TOTA.GetText(Stream: TMemoryStream);
const
  BuffLen = 16383;
var
  TextBuffer: PChar;
  Readed, BuffPos: LongInt;
begin
  Stream.Clear;
  GetMem(TextBuffer, BuffLen + 1);
  BuffPos := 0;
  try
    repeat
      Readed := GetText(BuffPos, TextBuffer, Bufflen);
      Stream.Write(TextBuffer^, Readed);
      inc(BuffPos, Readed);
    until Readed < BuffLen;
  finally
    FreeMem(TextBuffer, BuffLen + 1);
  end;
end;

class procedure TOTA.GetText(List: TStringList);
var
  Stream: TMemoryStream;
begin
  Stream := TMemoryStream.Create;
  GetText(Stream);
  List.SetText(Stream.Memory);
  Stream.Free;
end;

class function TOTA.GetTextLen: LongInt;
const
  BuffLen = 16383;
var
  TextBuffer: PChar;
  Readed, BuffPos: LongInt;
begin
  Result := 0;
  GetMem(TextBuffer, BuffLen + 1);
  BuffPos := 0;
  try
    repeat
      Readed := GetText(BuffPos, TextBuffer, Bufflen);
      Result := Result + Readed;
      inc(BuffPos, Readed);
    until Readed < BuffLen;
  finally
    FreeMem(TextBuffer, BuffLen + 1);
  end;
end;

class procedure TOTA.ReplaceText(NewText: PChar);
var
  TextLen: LongInt;
begin
  TextLen := GetTextLen;
  try
    CopyTo(0);
    DeleteTo(TextLen - 1);
    Insert(NewText);
  finally
  end;
end;

class function TOTA.GetEditor: TComponent;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to Application.ComponentCount - 1 do
  begin
    if pos('EditWindow_', Application.Components[I].Name) = 1 then
    begin
      Result := Application.Components[I].FindComponent('Editor');
      break;
    end;
  end;
  if Result = nil then raise exception.Create('Sorry, no EditWindow available');
end;

class function TOTA.GetCaretX: Integer;
var
  aEditor: TComponent;
begin
  aEditor := GetEditor;
  Result := GetOrdProp(aEditor, TmProp.GetPropByName(aEditor, 'CaretX'));
end;

class function TOTA.GetCaretY: Integer;
var
  aEditor: TComponent;
begin
  aEditor := GetEditor;
  Result := GetOrdProp(aEditor, TmProp.GetPropByName(aEditor, 'CaretY'));
end;

class function TOTA.GetCharXIndex: Integer;
var
  aEditor: TComponent;
begin
  aEditor := GetEditor;
  Result := GetOrdProp(aEditor, TmProp.GetPropByName(aEditor, 'CharXIndex'));
end;

class function TOTA.GetLineText: string;
var
  aEditor: TComponent;
begin
  aEditor := GetEditor;
  Result := GetStrProp(aEditor, TmProp.GetPropByName(aEditor, 'LineText'));
end;

class function TOTA.GetWordAtCaret: string;
var
  aLineText: string;
  XPos: Integer;
  LineLen: Integer;
  I, J: Integer;
begin
  aLineText := GetLineText;
  XPos := GetCaretX;
  LineLen := Length(aLineText);
  Result:= '';
  J:= 0;
  if XPos <= LineLen then
  begin
    I:= XPos;
    if aLineText[I] in ['_', '0'..'9', 'A'..'Z', 'a'..'z'] then
    begin
      dec(I);
      while aLineText[I] in ['_', '0'..'9', 'A'..'Z', 'a'..'z'] do
      begin
        if I = 0 then break;
        dec(I);
      end;
      inc(I);
      J:= I;
      while aLineText[J] in ['_', '0'..'9', 'A'..'Z', 'a'..'z'] do
      begin
        inc(J);
        if J > LineLen then break;
      end;
    end;
    Result:= Copy(aLineText, I, J-I);
  end;
end;

class function TOTA.GetHotlinkText: string;
var
  aEditor: TComponent;
begin
  aEditor := GetEditor;
  Result := GetStrProp(aEditor, TmProp.GetPropByName(aEditor, 'HotlinkText'));
end;

class function TOTA.GetLineCount: Integer;
var
  aEditor: TComponent;
begin
  aEditor := GetEditor;
  Result := GetOrdProp(aEditor, TmProp.GetPropByName(aEditor, 'LineCount'));
end;

class function TOTA.GetLinesInWindow: Integer;
var
  aEditor: TComponent;
begin
  aEditor := GetEditor;
  Result := GetOrdProp(aEditor, TmProp.GetPropByName(aEditor, 'LinesInWindow'));
end;

class function TOTA.GetSearchAfter: Integer;
var
  aEditor: TComponent;
begin
  aEditor := GetEditor;
  Result := GetOrdProp(aEditor, TmProp.GetPropByName(aEditor, 'SearchAfter'));
end;

class function TOTA.GetSearchBegin: Integer;
var
  aEditor: TComponent;
begin
  aEditor := GetEditor;
  Result := GetOrdProp(aEditor, TmProp.GetPropByName(aEditor, 'SearchBegin'));
end;

class function TOTA.GetTopLine: Integer;
var
  aEditor: TComponent;
begin
  aEditor := GetEditor;
  Result := GetOrdProp(aEditor, TmProp.GetPropByName(aEditor, 'TopLine'));
end;

end.

